﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Description;
using System.ServiceModel;
using Artech.ServiceInvocationViaMetadata.Contracts;
using System.Xml;

namespace Client
{
    class Program
    {
        static void Main(string[] args)
        {
            MetadataExchangeClient metadataExchangeClient = new MetadataExchangeClient(MetadataExchangeBindings.CreateMexHttpBinding());
            MetadataSet metadata = metadataExchangeClient.GetMetadata(new EndpointAddress("http://127.0.0.1:3721/calculatorservice/mex"));
            WsdlImporter wsdlImporter = new WsdlImporter(metadata);
            ContractDescription contract = ContractDescription.GetContract(typeof(ICalculator));
            wsdlImporter.KnownContracts.Add(new XmlQualifiedName(contract.Name, contract.Namespace), contract);
            ServiceEndpointCollection endpoints = wsdlImporter.ImportAllEndpoints();
            using (ChannelFactory<ICalculator> channelFactory = new ChannelFactory<ICalculator>(endpoints[0]))
            {
                ICalculator calculator = channelFactory.CreateChannel();
                using (calculator as IDisposable)
                {
                    try
                    {
                        Console.WriteLine("x + y = {2} when x = {0} and y = {1}", 1, 2, calculator.Add(1, 2));
                    }
                    catch(TimeoutException)
                    {
                        (calculator as ICommunicationObject).Abort();
                        throw;
                    }
                    catch(CommunicationException)
                    {
                        (calculator as ICommunicationObject).Abort();
                        throw;
                    }
                }
            }

            Console.Read();
        }
    }
}
